package utils;

use DBI;

use vars qw(
   @ISA @EXPORT
   @EXPORT_OK
   %EXPORT_TAGS
   $dbh
   );

use Exporter;

@ISA = qw(Exporter);

@EXPORT = qw();
@EXPORT_OK = qw(
  ReadParse 
  url_encode
  url_decode 
  report_bad_access
  curr_date
  find_date
  unix_date
  logit
  commify
  );

%EXPORT_TAGS = (all => [@EXPORT_OK] );

sub ReadParse {
   local (*in) = @_ if @_;
   local ($i, $key, $val);

   if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
      $in = $ENV{'QUERY_STRING'};
   } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
      read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
   } else {
      # Added for command line debugging
      # Supply name/value form data as a command line argument
      # Format: name1=value1\&name2=value2\&...
      # (need to escape & for shell)
      # Find the first argument that's not a switch (-)
      $in = ( grep( !/^-/, @ARGV )) [0];
      $in =~ s/\\&/&/g;
   }

   @in = split(/&/,$in);

   foreach $i (0 .. $#in) {
      # Convert plus's to spaces
      $in[$i] =~ s/\+/ /g;

      # Split into key and value.
      ($key, $val) = split(/=/,$in[$i],2); # splits on the first =.

      # Convert %XX from hex numbers to alphanumeric
      $key =~ s/%(..)/pack("c",hex($1))/ge;
      $val =~ s/%(..)/pack("c",hex($1))/ge;

      # Associate key and value. \0 is the multiple separator
      $in{$key} .= "\0" if (defined($in{$key}));
      $in{$key} .= $val;
      }
   return length($in);
}


#encode a sql statement to be passed to another form. The order of these transformations is important
sub url_encode {
    my $text = shift;
    my @Encode_Chars = ('\%', '\+', '\;', '\,', '\=', '\&', '\:\:', '\s');

    my %Encode_Chars = (
      '\%',   '%25',
      '\+',   '%2B',
      '\;',   '%3B',
      '\,',   '%2C',
      '\=',   '%3D',
      '\&',   '%26',
      '\:\:', '%3A%3A',
      '\s',   '+');

     $text =~ s/'/sqt/g;
     $text =~ s/>/gtchar/g;
     $text =~ s/</ltchar/g;
     $text =~ s/\(/leftparen/g;
     $text =~ s/\)/rightparen/g;

     foreach $char (@Encode_Chars) {
         $text =~ s/$char/$Encode_Chars{$char}/g;
     }
    
    #$text =~ s/%(.{2})/chr(hex($1))/eg;
    #$text =~ s/([^a-zA-Z0-9_.!~*'( )-])/sprintf "%%%02X", ord($1)/egi;
    return $text;
}

#decode the sql passed in from another form to make it usable
sub url_decode {
    my $text = shift;
   @Decode_Chars = ('\+', '\%3A\%3A', '\%26', '\%3D', '\%2C', '\%3B', '\%2B', '\%25');

   %Decode_Chars = (
   '\+',       ' ',
   '\%3A\%3A', '::',
   '\%26',     '&',
   '\%3D',     '=',
   '\%2C',     ',',
   '\%3B',     ';',
   '\%2B',     '+',
   '\%25',     '%'
   );

   #$text =~ s/%(..)/pack("c",hex($1))/ge; 
   #$text =~ s/%([a-f0-9][a-f0-9])/chr( hex( $1 ) )/ei;
   foreach $char (@Decode_Chars) {
       $text =~ s/$char/$Decode_Chars{$char}/g;
   }

   $text =~ s/sqt/\'/g;
   $text =~ s/gtchar/\>/g;
   $text =~ s/ltchar/\</g;
   $text =~ s/leftparen/\(/g;
   $text =~ s/rightparen/\)/g;
   return $text;
}

#pass in the access flag and the pageid, will print the web text to the user about denied access
sub report_bad_access {
   my ($accessflag,$pageid)=@_;
   
   if ($accessflag != 1) {
      print "Content-type: text/html", "\n\n";
      print qq{
      <head>
      <style>
      body {
         margin-left:200px;
      }
      </style>
      <br><br>
      <h3>You do not have access to this document.. Please contact the system administrator..</h3>
      <br><strong>Information:</strong>
      <br>accessflag ($accessflag)  
      <br>username='$ENV{HTTP_SM_LAST_NAME},$ENV{HTTP_SM_FIRST_NAME}' 
      <br>pageid='$pageid'
      <br>
      <br>Contact Bobby J George for any access requests -  972.652.5028
      };
      exit;
   }

}#sub bad_access

#return a date offset from a specified UNIX date in a specified format
sub find_date {
   my ($origdate,$offset,$format) = @_;
   #set a default date format
   $format=$format||"yyyymmdd";
   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
 
   #add $offset days worth of seconds to the unix integer date
   #convert the date back to an english string and parse it
   my $newdate=localtime(($origdate+($offset*86400)));
   
   my ($nmonth,$nday,$nyear)=($newdate)=~/\w+\s+(\w{3})\s+(\d+)\s+.+(\d{4})/;
   $nday=sprintf("%02d",$nday);
   $format=~s/[yY]+/$nyear/;
   $format=~s/[mM]+/$month_num{$nmonth}/;
   $format=~s/[dD]+/$nday/;
   return "$format";
}

#return a yyyymmdd datestring from the current date (in yyyymmdd format)
sub curr_date {
   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
   #get the current unix integer time
   #convert the current time to an english string and parse it
   my $currdate=localtime(time());
   my ($cmonth,$cday,$cyear)=($currdate)=~/\w+\s+(\w{3})\s+(\d+)\s+.+(\d{4})/;
   $cday=sprintf("%02d",$cday);
   return "$cyear$month_num{$cmonth}$cday";
}

# return a UNIX integer date from a datestring.  
# Accepts yyyymmdd, and mm/dd/yyyy formats.  hours, mins, secs are not used.
sub unix_date {
   my ($indate)=@_;
   my ($mm,$dd,$yyyy,$timestamp);
   #accept the format mm/dd/yyyy
   if ($indate=~/^\d{2}\/\d{2}\/\d{4}$/) {
      ($mm,$dd,$yyyy)=$indate=~/^(\d{2})\/(\d{2})\/(\d{4})$/;
   }elsif  ($indate=~/^\d{8}$/) {
      #default format yyyymmdd
      ($yyyy,$mm,$dd)=$indate=~/^(\d{4})(\d{2})(\d{2})$/;
   }
   use POSIX;
   $mm--;
   $yyyy-=1900;
   $timestamp = mktime(0,0,0,$dd,$mm,$yyyy,0,0,-1);
   return "$timestamp";
}

sub logit {
   my ($logaction,$string)=@_;
   return if (!$logaction);
   local $\="\n"; # set output record separator
   
   #use the ENV{logfile} variable for the file if it is defined.
   $logfile=$ENV{logfile}||(split("/",$0))[-1].".log";

   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
   #split the date string into the separate pieces
   my($cmonth,$cday,$ctime,$cyear)= localtime(time()) =~ /^.{3}\s(.{3})\s(.{2})\s(.{8})\s(.{4})$/;
   $cday=sprintf("%02d",$cday);
   $cdate = "$cyear$month_num{$cmonth}$cday";
   my $datestring="$cdate $ctime";
   
   if (!$string && $logaction !~/^init$|^delete$/i) {
      $string=$logaction;
      $logaction="print";
   }
   my $status;
   switch: for ($logaction) {
      /^init$/i && do {
         $status=open(outfile, ">/tmp/$logfile") || die "Cant open file - (/tmp/$logfile)";
         $status=print outfile "\n$datestring  Logfile Created.";
         close outfile;
         system("chmod 777 /tmp/$logfile");
         last switch;
      };
      /^print$/i && do {
         if (! -f "/tmp/$logfile") {
            system("touch /tmp/$logfile");
            system("chmod 777 /tmp/$logfile");
         }
         open(outfile, ">>/tmp/$logfile") || die "Cant open file - (/tmp/$logfile)";
         $status=print outfile "$datestring $string";
         close outfile;
         last switch;
      };
      /^delete$/i && do {
         $status=1;
         if (-f "/tmp/$logfile") {
            $status=unlink("/tmp/$logfile");
         }
         last switch;
      };
   }#switch
   return ($status);
}#sub logit

sub commify {
   local $_  = shift;
   1 while s/^([-+]?\d+)(\d{3})/$1,$2/;
   return $_;
}

1;

__END__
