#!/usr/bin/env perl
##################################################################################################
#
# actions script to support xmasviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

use CGI;
use CGI::Carp qw(fatalsToBrowser);

use lib "$ENV{SERVERTOP}/tools/lib";#path to utils.pm.  Optional if utils.pm is in current dir
use utils qw(logit);
use DBI;

use Config::Simple;

#read in a simple config file to get the db access parameters
my $cfg = new Config::Simple('xmasviewer.conf');
my $db_source = $cfg->param('db_source');
my $db_user = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug = $cfg->param('debug') || 0;
my $db_switch = $cfg->param('db_switch');


my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

&logit(qq{\n\nprocessing parameters:}) if $debug;
while (($holder,$value)=each (%params)) {
   &logit(qq{   $holder = $value}) if $debug;
}


my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $db_source: $DBI::errstr";

#build a structure so I can look up the id of the recipient when it isnt modified on the row
my %recepientmap;
$sql = qq{SELECT
    xmas_recipients.name,
    xmas_recipients.id
FROM
    xmas_recipients};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$recipient,\$id);

while ($sth->fetch()) {
   $recepientmap{$recipient}=$id;
}

#build a structure so I can look up the id of the status when it isnt modified on the row
my %statusmap;
$sql = qq{SELECT
    xmas_status.name,
    xmas_status.id
FROM
    xmas_status};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$status,\$id);

while ($sth->fetch()) {
   $statusmap{$status}=$id;
}


my @updaterows = split(',',$params{ids});
&logit(qq{ updaterows   @updaterows}) if $debug;

my $resultString = qq{<data>};

#0 -> id
#1 -> season
#2 -> recipient
#3 -> item
#4 -> description
#5 -> price
#6 -> status
#7 -> delete checkbox

foreach $rowid (@updaterows) {

   my $itemidkey = qq{$rowid\_c0};
   my $itemid = $params{$itemidkey};
   &logit(qq{ itemidkey $itemid}) if $debug;

   my $seasonkey = qq{$rowid\_c1};
   my $season = $params{$seasonkey};
   &logit(qq{ season $season}) if $debug;

   my $recipientkey = qq{$rowid\_c2};
   my $recipient = $params{$recipientkey};
   if ($recipient !~ /^\d+$/) {
      $recipient = $recepientmap{$recipient};
   }
   &logit(qq{ recipient $recipient}) if $debug;

   my $itemnamekey = qq{$rowid\_c3};
   my $itemname = $params{$itemnamekey};
   &logit(qq{ itemname $itemname}) if $debug;

   my $itemdesckey = qq{$rowid\_c4};
   my $itemdesc = $params{$itemdesckey};
   &logit(qq{ itemdesc $itemdesc}) if $debug;

   my $itempricekey = qq{$rowid\_c5};
   my $itemprice = $params{$itempricekey};
   &logit(qq{ itemprice $itemprice}) if $debug;

   my $statusidkey = qq{$rowid\_c6};
   my $statusid = $params{$statusidkey};
   if ($statusid !~ /^\d+$/) {
      $statusid = $statusmap{$statusid};
   }
   &logit(qq{ statusid $statusid}) if $debug;

   my $deletekey = qq{$rowid\_c7};
   my $delkey = $params{$deletekey};
   &logit(qq{ deletekey $deletekey}) if $debug;

   #!nativeeditor_status = inserted 
   my $rowstatusidkey = qq{$rowid\_!nativeeditor_status}; 
   my $rowstatuskey = $params{$rowstatusidkey};
   &logit(qq{ rowstatuskey $rowstatuskey}) if $debug;

   if ($rowstatuskey eq "inserted") {
      &logit(qq{ new record to insert}) if $debug;
      # id season name price description receiver status modified
      my $sql = qq{insert into xmas_items(season,name,price,description,receiver,status) values ($season,'$itemname',$itemprice,'$itemdesc',$recipient,$statusid)};
      &logit(qq{ insert sql - $sql}) if $debug;
      $dbh->do($sql);
      $resultString .= qq{\n<action type='inserted' sid='$rowid' tid='$rowid'/>};
   }elsif ($rowstatuskey eq "updated") {
      if ($delkey == 1) {
         my $sql = qq{delete from xmas_items where id= $itemid};
         &logit(qq{ delete sql - $sql}) if $debug;
         $dbh->do($sql);
         $resultString .= qq{\n<action type='deleted' sid='$rowid' tid='$rowid'/>};
      }else{
         my $sql = qq{update xmas_items set season=$season,name='$itemname',price=$itemprice,description='$itemdesc',receiver=$recipient,status=$statusid where id = $itemid}; 
         &logit(qq{ update sql - $sql}) if $debug;
         $dbh->do($sql);
         $resultString .= qq{\n<action type='updated' sid='$rowid' tid='$rowid'/>};
      }
   }#elsif
};

$resultString .= qq{\n</data>};

&logit(qq{$resultString}) if $debug;


print "Content-type: text/xml", "\n\n";
print qq{<?xml version="1.0"  encoding="ISO-8859-1"?>};

print qq{
$resultString
};
