#!/usr/bin/env perl
##################################################################################################
#
# actions script to support xmasviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

#use xoutils qw(logit dbconnect_lookup);
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;

use lib "$ENV{SERVERTOP}/tools/lib"; #path to utils.pm.  Optional if utils.pm is in current dir
use utils qw(logit commify);


#read in a simple config file to get the db access parameters
my $cfg = new Config::Simple('xmasviewer.conf');
my $db_source = $cfg->param('db_source');
my $db_user = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug = $cfg->param('debug') || 0;
my $db_switch = $cfg->param('db_switch');

&logit(qq{(db_source,db_user,db_password ($db_source,$db_user,$db_password)}) if $debug;

my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

&logit(qq{\n\nprocessing parameters:}) if $debug;
while (($holder,$value)=each (%params)) {
   &logit(qq{   $holder = $value}) if $debug;
}

#-----------------------------------------------------------------------------------------------------
#Fetch Notes
if ($params{action} eq "fetchnotes") {
   &logit(qq{   doing fetchnotes()}) if $debug;
   
   my $sql = qq{
   SELECT
      xmas_usernotes.notes
   FROM
      xmas_usernotes
   WHERE
      xmas_usernotes.userid=$params{recipientid}
   };

   &logit(qq{sql:\n$sql}) if $debug;

   my $usernotes;

   my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $data_source: $DBI::errstr";
   $sth = $dbh->prepare($sql);
   $sth->execute();
   $sth->bind_columns(\$usernotes);
   $sth->fetch();
   print "Content-type: text/html", "\n\n";
   $usernotes = ($usernotes =~ /\w/  ? $usernotes : qq{<b>Needs:</b><br><br><br><b>Doesnt need:</b><br><br><br><b>Likes:</b><br><br><br><b>Dislikes:</b><br><br>});
   print qq{$usernotes};
   exit;
}

#-----------------------------------------------------------------------------------------------------
#Set Notes
if ($params{action} eq "setnote") {
   &logit(qq{   doing setnote()}) if $debug;
   my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $data_source: $DBI::errstr";
   my $delsql = qq{DELETE from xmas_usernotes where xmas_usernotes.userid=$params{recipientid}};
   &logit(qq{delsql:\n$delsql}) if $debug;
   
   $dbh->do($delsql);
   my $sql = qq{
   INSERT
   INTO
      xmas_usernotes
      (
         userid,
         notes
      )
      VALUES
      (
         $params{recipientid},
         '$params{notetext}'
      )
   };

   &logit(qq{sql:\n$sql}) if $debug;
   $dbh->do($sql);
   print "Content-type: text/html", "\n\n";
   print qq{Done};
   exit;
}


#-----------------------------------------------------------------------------------------------------
#search for Items
&logit(qq{   doing dbsearch()}) if $debug;

my $sql = qq{
SELECT
    xmas_items.id,
    xmas_items.season,
    xmas_items.name,
    xmas_items.description,
    xmas_items.price,
    xmas_items.price,
    xmas_recipients.name,
    upper(xmas_status.name),
    xmas_status.color
FROM
    xmas_items
INNER JOIN
    xmas_status
ON
    (
        xmas_items.status = xmas_status.id)
INNER JOIN
    xmas_recipients
ON
    (
        xmas_items.receiver = xmas_recipients.id)
WHERE 1 = 1
};

#order by modified_date DESC 
#limit 100
#
if ($params{fromdate} && $params{todate}) {
   #handle the issue if the user sets and end date earlier than the start date
   my @dateslist;
   push(@dateslist,$params{fromdate});
   push(@dateslist,$params{todate});
   @dateslist = sort { $b cmp $a } @dateslist;
   &logit(qq{   \@dateslist = @dateslist}) if $debug;
   $params{fromdate} = pop(@dateslist);
   $params{todate} = pop(@dateslist);
}
$sql .= qq{\n and xmas_items.receiver = $params{recipientid}} if ($params{recipientid});
$sql .= qq{\n and xmas_items.season = $params{season}} if ($params{season});
# $sql .= qq{\n and xmas_status.id = '$params{statusid}'} if ($params{statusid});
if ($params{statusid}) {
   if ($params{statusid} == 999) {
      #the 'all but acquired' is selected
      $sql .= qq{\n and xmas_status.id != 3};
   }else{
      $sql .= qq{\n and xmas_status.id = $params{statusid}};
   }
}
$sql .= qq{\n and xmas_items.name like '\%$params{likestring}\%'} if ($params{likestring});

$sql .= qq{\n order by xmas_items.season DESC,xmas_recipients.name ASC};

&logit(qq{sql:\n$sql}) if $debug;

my $resultString;

my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $data_source: $DBI::errstr";
$sth = $dbh->prepare($sql);
$sth->execute();

my $rowcount=0;
my $grandtotal=0;
$sth->bind_columns(\$itemid,\$season,\$itemname,\$itemdesc,\$dispitemprice,\$itemprice,\$recipient,\$status,\$statuscolor);
my $resultString;
while ($sth->fetch()) {
$resultString .= qq{\n<row id='$rowcount'>
<cell>$itemid</cell>
<cell>$season</cell>
<cell>$recipient</cell>
<cell>$itemname</cell>
<cell>$itemdesc</cell>
<cell>$dispitemprice</cell>
<cell style="color:$statuscolor">$status</cell>
<cell>0</cell>
</row>};
$rowcount++;
$grandtotal += $itemprice;
}

$resultString =~ s/\&/\&amp;/g;
$resultString =~ s/\n//g;
$resultString =~ s/\r//g;
&logit(qq{\n\nresults:\n$resultString}) if $debug;
print "Content-type: text/xml", "\n\n";
print qq{<?xml version="1.0"  encoding="ISO-8859-1"?><rows total_count='$rowcount' pos='0'>$resultString</rows>};
#print q{<?xml version="1.0" encoding="ISO-8859-1"?><rows><row id='0'><cell>AMA</cell><cell>5BA</cell><cell>2011-12-27 15:49:19</cell><cell>1439.965000</cell><cell></cell><cell>1439.96500</cell><cell>4107886300</cell><cell></cell><cell></cell><cell>4435246245</cell><cell>0000</cell><cell></cell><cell>0</cell><cell>1</cell><cell>4435240000</cell><cell></cell><cell></cell><cell>2010-11-13 00:30:45</cell><cell></cell></row></rows>};
