#!/usr/bin/env perl
##################################################################################################
#
# summary report for  xmasviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;

use lib "$ENV{SERVERTOP}/tools/lib"; #path to utils.pm.  Optional if utils.pm is in current dir
use utils qw(logit commify);

#read in a simple config file to get the db access parameters
my $cfg = new Config::Simple('xmasviewer.conf');
my $db_source = $cfg->param('db_source');
my $db_user = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug = $cfg->param('debug') || 0;
my $db_switch = $cfg->param('db_switch');

my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $data_source: $DBI::errstr";

my %totalshash;
my @recipientlist;
my %grandtotalshash;
my %seasonhash;

#build a structure so I can look up the id of the status when it isnt modified on the row
my %statusmap;
$sql = qq{SELECT
    xmas_status.name,
    xmas_status.id
FROM
    xmas_status
order by xmas_status.id};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$status,\$id);

while ($sth->fetch()) {
   $statusmap{$id}=$status;
}

my $sql = qq{
SELECT
    xmas_recipients.name
FROM
    xmas_recipients
WHERE  xmas_recipients.name != 'Unassigned'
ORDER BY
    xmas_recipients.disp_order,xmas_recipients.name
};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$recipient);
while ($sth->fetch()) {
   $totalshash{$recipient} = 0;
   push(@recipientlist,$recipient);
}

my $sql = qq{
SELECT
    xmas_items.season,
    xmas_recipients.name,
    xmas_items.price,
    xmas_items.status
FROM
    xmas_recipients
LEFT OUTER JOIN
    xmas_items
ON
    (
        xmas_recipients.id = xmas_items.receiver)
where status=3
and xmas_recipients.name != 'Unassigned'
and xmas_items.season > year(date_sub(curdate(), interval 5 year))
order by season desc
};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$season,\$recipient,\$price,\$status);

while ($sth->fetch()) {
   $totalshash{$season}{$recipient}+=$price;
   $grandtotalshash{$season}+=$price;
   my $statusname = $statusmap{$status};
   $seasonhash{$season}++;
}

#totalshash{2011}{bob}=200

#get a list of all the seasons in the db
my @seasonlist = sort { lc($b) cmp lc($a) }keys %seasonhash;
my @statuslist = (sort keys %statusmap);

#for the alternating color rows
my @colors=( '#e6e6e6' ,'#FFFFFF');

my $rangerowguts;
my $headerrowguts;
my $tableguts;
for ($t=0; $t<=scalar(@seasonlist); $t++) {
   my $season = $seasonlist[$t];
   next if ($season =~ /^ *$/);
   $rangerowguts .= qq{<th colspan=2>$season</th>};
   $headerrowguts .= qq{<th>Recipient</th><th>Total</th>};
}

for ($x=0; $x<=scalar(@recipientlist); $x++) {
   my $tablerow;
   my $user = $recipientlist[$x];
   next if ($user =~ /^ *$/);
   for ($t=0; $t<=scalar(@seasonlist); $t++) {
      my $season = $seasonlist[$t];
      next if ($season =~ /^ *$/);
      my $usertotal = int($totalshash{$season}{$user});
      my $colcolor = $colors[ $t % 2 ];
      $tablerow .= qq{<td style="background-color:$colcolor;">$user</td><td style="text-align:right;background-color:$colcolor;">\$$usertotal</td>};
   }
   $tableguts .= qq{<tr>$tablerow</tr>};
}

my $totalrow;
for ($t=0; $t<=scalar(@seasonlist); $t++) {
   my $season = $seasonlist[$t];
   next if ($season =~ /^ *$/);
   my $seasontotal = $grandtotalshash{$season};
   $seasontotal = &commify($seasontotal);
   my $colcolor = $colors[ $t % 2 ];
   $totalrow .= qq{<td style="font-weight:bold;background-color:$colcolor;">Total</td><td style="text-align:right;font-weight:bold;background-color:$colcolor">\$$seasontotal</td>};
}
$tableguts .= qq{<tr>$totalrow</tr>};

my $newtabledata = qq{
<table class="summarytable">
<tr>$rangerowguts</tr>
<tr>$headerrowguts</tr>
$tableguts
</table>
};

print "Content-type: text/html", "\n\n";

print qq{
<html>
<head>
<link rel="stylesheet" type="text/css" href="/css/xmasviewer.css">
</head>
<body>
<center>
<h3>Christmas 5 Year Summary</h3>
$newtabledata
<br>
Note: Only acquired items are reflected in totals
<br><br>
<a href="xmasviewer.cgi">Christmas List Manager</a>
</body>
</html>
};
