#!/usr/bin/env perl
##################################################################################################
#
# actions totals script to support xmasviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;

use lib "$ENV{SERVERTOP}/tools/lib"; #path to utils.pm.  Optional if utils.pm is in current dir
use utils qw(logit commify);

#read in a simple config file to get the db access parameters
my $cfg = new Config::Simple('xmasviewer.conf');
my $db_source = $cfg->param('db_source');
my $db_user = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug = $cfg->param('debug') || 0;
my $db_switch = $cfg->param('db_switch');

&logit(qq{(db_source,db_user,db_password ($db_source,$db_user,$db_password)}) if $debug;

my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

&logit(qq{\n\nprocessing parameters:}) if $debug;
while (($holder,$value)=each (%params)) {
   &logit(qq{   $holder = $value}) if $debug;
}

my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $data_source: $DBI::errstr";

my %totalshash;
my $grandtotal=0;
my @totals;
my %statushash;
my %donehash;
my $statusline;
my $itemscount=0;

#set the done status for an individual and return
if ($params{action} eq "setdone") {
	my $season     = $params{season};
	my $userid     = $params{userid};
	my $donestatus = $params{donestatus};
	if ($season == 999 || $season =~ /^ *$/) {
	   return 1;
	}
	my $sql = qq{delete from xmas_recipients_done where userid=$userid and year=$season};
	&logit(qq{ delete sql - $sql}) if $debug;
	$dbh->do($sql);

	my $sql = qq{INSERT INTO `xmas_recipients_done`(`userid`, `year`, `done`) VALUES ($userid,$season,$donestatus)};
	&logit(qq{ insert sql - $sql}) if $debug;
	$dbh->do($sql);
	print "Content-type: text/html", "\n\n";
	exit 1;
}


#build a structure so I can look up the id of the status and color when it isnt modified on the row
my %statusmap;
my %colormap;
$sql = qq{SELECT
    xmas_status.name,
    xmas_status.id,
    xmas_status.color
FROM
    xmas_status
order by xmas_status.id};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$status,\$id,\$color);

while ($sth->fetch()) {
   $statusmap{$id}=$status;
   $colormap{$id}=$color;
}

##   get the list of users and create the totals hash
my @ordereduserlist;
my $sql = qq{
SELECT
    xmas_recipients.name,
    xmas_recipients.id
FROM
    xmas_recipients
ORDER BY
    xmas_recipients.disp_order,xmas_recipients.name
};

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$recipient,\$recipientid);
while ($sth->fetch()) {
   $totalshash{$recipient} = 0;
   push(@ordereduserlist,"$recipient:$recipientid");
}

if ($params{season}) {
	my $searchyear = $params{season};
	#get the done status for each user
	my $sql = qq{
SELECT
		xmas_recipients.name,
		xmas_recipients.id,
		xmas_recipients_done.done
	FROM
		xmas_recipients,
		xmas_recipients_done
	WHERE 
		xmas_recipients.id = xmas_recipients_done.userid 
		AND xmas_recipients_done.year = $searchyear
		AND xmas_recipients_done.done = 1
	};
	$sth = $dbh->prepare($sql);
	$sth->execute();
	$sth->bind_columns(\$rec,\$recid,\$doneval);
	while ($sth->fetch()) {
	   $donehash{$searchyear}{"$rec:$recid"} = $doneval;
	}
}

## populate the totals for each user
while (($holder,$value)=each (%totalshash)) {
   &logit(qq{$holder=($value)}) if $debug;
}

my $sql = qq{
SELECT
    xmas_recipients.name,
    xmas_items.price,
    xmas_items.status
FROM
    xmas_recipients
LEFT OUTER JOIN
    xmas_items
ON
    (
        xmas_recipients.id = xmas_items.receiver)
WHERE
    1 = 1
};
#and xmas_recipients.name != 'Unassigned'

$sql .= qq{\n and xmas_items.season = $params{season}} if ($params{season});
$sql .= qq{\n and xmas_items.receiver = $params{recipientid}} if ($params{recipientid});
$sql .= qq{\n and xmas_items.name like '\%$params{likestring}\%'} if ($params{likestring});

if ($params{statusid}) {
   if ($params{statusid} == 999) {
      #the 'all but acquired' is selected
      $sql .= qq{\n and xmas_items.status != 3};
   }else{
      $sql .= qq{\n and xmas_items.status = $params{statusid}};
   }
}

my $seasonstring = $params{season} || "All";

&logit(qq{sql:\n$sql}) if $debug;

$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$recipient,\$price,\$status);

while ($sth->fetch()) {
   #push(@totals,qq{<td>$recipient</td><td style="text-align:right;">$total</td>});

   $totalshash{$recipient}+=$price;
   my $statusname = $statusmap{$status};
   $statushash{"$statusname"}++;
   $itemscount++;
   #skip planned merchandise.  totals only reflect actuly spent dollars
   if ($statusname !~ /planned/i) {
       $grandtotal += $price;
   }
}

$grandtotal = &commify($grandtotal);
my $recipientcount = scalar(@ordereduserlist);
$statusline = qq{Season : $seasonstring&nbsp;&nbsp;&nbsp;&nbsp;Recipients : $recipientcount&nbsp;&nbsp;&nbsp;&nbsp;Gifts : $itemscount&nbsp;&nbsp;&nbsp;&nbsp;-};

my @statuslist = (sort keys %statusmap);
for ($t=0; $t<=scalar(@statuslist); $t++) {
#foreach (values %statusmap) {
   my $statusname = $statusmap{$t};
   my $statuscolor = $colormap{$t};
   next if ($statusname =~ /^ *$/);
   my $value = int($statushash{$statusname});
   $statusline .= qq{&nbsp;&nbsp;&nbsp;&nbsp;<span style="color:$statuscolor">$statusname : $value</span>};
}

my $totaltable = qq{
$statusline
<table id="totaltable">
   <tr>
      <th>Recipient</th>
      <th>Total</th>
      <th>Recipient</th>
      <th>Total</th>
      <th>Recipient</th>
      <th>Total</th>
      <th>Recipient</th>
      <th>Total</th>
      <th>Recipient</th>
      <th>Total</th>
   </tr>};

#build a five column table of the totals
for ($t=0; $t<=scalar(@ordereduserlist); $t++) {
    $totaltable .= qq{\n   <tr>};
    for ($x = 0; $x <= 4; $x++) {
		my ($user,$userid)  = split(':',$ordereduserlist[$t+$x]);
		my $doneval = ($donehash{$params{season}}{$ordereduserlist[$t+$x]} == 1 ? 'checked' : '');
		
		#empty user  - fill out the table td with a space to look right
		if ($user =~ /^ *$/) {
		    $totaltable .= qq{<td>&nbsp;</td><td>&nbsp;</td>};
		}else{ 
		    #Only display the done checkboxes when the Recipient search is not 'Unassigned' the the Season isnt 'Unassigned' or 'All' 
		    if ($params{recipientid} == 999 || $user =~ /Unassigned/i || $params{season} == 9999 || $params{season} =~ /^ *$/i) {
			    $totaltable .= qq{<td>$user</td><td style="text-align:right;">\$$totalshash{$user}</td>};
		    }else{
			    $totaltable .= qq{<td><input type=checkbox id="$ordereduserlist[$t+$x]_done" $doneval onclick="setDoneStatus(this,'$userid','$params{season}');" >};
			    $totaltable .= qq{<label for="$ordereduserlist[$t+$x]_done">$user</label></td>};
			    $totaltable .= qq{<td style="text-align:right;">\$$totalshash{$user}</td>};
		    }
		}
    }
    $totaltable .= qq{\n</tr>};
    $t+=4;
}

#finish up the table with a footer
$totaltable .= qq{
<tr><td style="text-align:right;" colspan=8>(Total excludes planned items)</td><td style="font-weight:bold;">Total</td><td style="font-weight:bold;text-align:right;">\$$grandtotal</td></tr>
</table>\
<span class="done_label">Checked = Done</span>&nbsp;&nbsp;&nbsp;<span class="undone_label">Unchecked = Not Done</span>
};

# $resultString =~ s/\&/\&amp;/g;
# $resultString =~ s/\n//g;
# $resultString =~ s/\r//g;

&logit(qq{\n\nresults:\n$totaltable}) if $debug;

#return the results
print qq{
$totaltable
};

exit 1;


