#!/usr/bin/env perl
##################################################################################################
#
# Frontend for the Christmas database
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#

$|=1;    # set output buffering to off
$,=" ";  # set output field separator

BEGIN {
    my $b__dir = (-d '/home/mastscr9/perl'?'/home/mastscr9/perl':( getpwuid($>) )[7].'/perl');
    unshift @INC,$b__dir.'5/lib/perl5',$b__dir.'5/lib/perl5/x86_64-linux-thread-multi',map { $b__dir . $_ } @INC;
}

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;

use lib "$ENV{SERVERTOP}/tools/lib";#path to utils.pm.  Optional if utils.pm is in current dir
use utils qw(logit);

#read in a simple config file to get the db access parameters
my $cfg = new Config::Simple('xmasviewer.conf');
my $db_source = $cfg->param('db_source');
my $db_user = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug = $cfg->param('debug') || 0;
my $db_switch = $cfg->param('db_switch');

&logit(qq{(db_source,db_user,db_password ($db_source,$db_user,$db_password)}) if $debug;

#establish a connection to the db
my $dbh = DBI->connect($db_source,$db_user,$db_password) || die "Can't connect: $DBI::errstr\n";

#-----------------------------------------------------------------------------------------------------
#season option menu (next year plus last 5 years)

my $currdate;
my @datelist;

my $seasonoptionpopulate; #for the grid popup
my $seasonoptionmenuguts; #for the search form

$seasonoptionmenuguts .= qq{<option value="9999">Unassigned</option>\n};
$seasonoptionpopulate .= qq{\nseasoncombo.put('9999','Unassigned');};

my $curryear = (localtime())[5];
my $fullcurryear = ($curryear + 1900);

for (-1..4) {
   my $calcyear = (($curryear-$_)+1900);
   push(@datelist,"$calcyear");
}
for ($t=0; $t<scalar(@datelist); $t++) {
   next if $datelist[$t] =~ /^ *$/;
   my $selectswitch = ""; 
   if ($t == 1) {
      $selectswitch = qq{selected};
   }
   $seasonoptionmenuguts .= qq{<option value="$datelist[$t]" $selectswitch>$datelist[$t]</option>\n};
   $seasonoptionpopulate .= qq{\nseasoncombo.put('$datelist[$t]','$datelist[$t]');};
}
$seasonoptionmenuguts .= qq{</select>};

#-----------------------------------------------------------------------------------------------------
# recipient option menu
my $recipientoptionpopulate;
my $recipientoptionmenuguts;
my $contextsql = qq{
SELECT
    id,
    name
FROM
    xmas_recipients
ORDER BY
    disp_order,name
};

$sth = $dbh->prepare($contextsql);
$sth->execute();
$sth->bind_columns(\$val,\$label);
while ( $sth->fetch() ) {
    $recipientoptionmenuguts .= qq{<option value="$val">$label</option>\n};
    $recipientoptionpopulate .= qq{\nrecipientcombo.put('$val','$label');};
}
$recipientoptionmenuguts .= qq{</select>};
$sth->finish();

$recipientoptionpopulate .= qq{\nrecipientcombo.put('999','Unassigned');};

#-----------------------------------------------------------------------------------------------------
# status option menu
my $statusoptionpopulate; 
my $statusoptionmenuguts; 
my $statsql = qq{
SELECT
    id,
    upper(name)
FROM
    xmas_status
ORDER BY
    id ASC
};

$sth = $dbh->prepare($statsql);
$sth->execute();
$sth->bind_columns(\$val,\$label);
while ( $sth->fetch() ) {
        $statusoptionmenuguts .= qq{<option value="$val">$label</option>\n};
        $statusoptionpopulate .= qq{\nstatcombo.put('$val','$label');};
}
#$statusoptionmenuguts .= qq{<option value="999">All But ACQUIRED</option>\n};
$statusoptionmenuguts .= qq{</select>};
$sth->finish();


 #    #  ######  #####   #####     ##     ####   ###### 
 #    #  #       #    #  #    #   #  #   #    #  #
 #    #  #####   #####   #    #  #    #  #       #####
 # ## #  #       #    #  #####   ######  #  ###  #
 ##  ##  #       #    #  #       #    #  #    #  #
 #    #  ######  #####   #       #    #   ####   ######

print "Content-type: text/html", "\n\n";

print qq{
<html>
<head>
<script src="/js/dhtmlxsuite/codebase/dhtmlx.js"></script>
<link rel="stylesheet" type="text/css" href="/js/dhtmlxsuite/skins/skyblue/dhtmlx.css"/>
<link rel="stylesheet" type="text/css" href="/css/xmasviewer.css">
<script src="/js/prototype.js"></script>

<title>Christmas List Manager</title>
<script language=Javascript>
function showMessage(message) {
   var span = document.getElementById("centermessagespan");
   span.innerHTML = message;
}
function showCount(originalRequest) {
   var returndata=originalRequest.responseText;
   document.getElementById("leftmsgspan").style.visibility='visible';
   document.getElementById("leftmsgspan").innerHTML=returndata+' Alerts';
}

function setDefaults() {
   document.getElementById("recipientid").value = '';
   document.getElementById("statusid").value = '';
   document.getElementById("season").value = $fullcurryear;
   document.getElementById("likestring").value = '';
   showMessage('');
}
function reloadGrid() {
   var recipientid = document.getElementById("recipientid").value;
   var statusid = document.getElementById("statusid").value;
   var likestring = document.getElementById("likestring").value;
   var season=document.getElementById("season").value;

   var params='recipientid='+recipientid+'&statusid='+statusid+'&likestring='+likestring+'&season='+season+'&action=search';

   mygrid.clearAll();

   var url = "xmas_search.cgi";

	mygrid.load(url+"?"+params,
   function(){
   //display the export message on the left
   document.getElementById("leftmsgspan").style.visibility='visible';
   //display the export message on the right
   document.getElementById("rightmsgspan").style.visibility='visible';
   //display the record count on the left
   document.getElementById("leftmsgspan").innerHTML=mygrid.getRowsNum()+' Items';
   },
   'xml');
   fetchtotals();
}
function fetchtotals(params) {
   var url = "xmas_totals.cgi";
   //var season=document.getElementById("season").value;
   //var params='season='+season;

   var recipientid = document.getElementById("recipientid").value;
   var statusid = document.getElementById("statusid").value;
   var likestring = document.getElementById("likestring").value;
   var season=document.getElementById("season").value;

   var params='recipientid='+recipientid+'&statusid='+statusid+'&likestring='+likestring+'&season='+season+'&action=search';
   //alert(params);
   var myAjax = new Ajax.Request(url,{method: 'post', parameters: params,onComplete: showtotals});
}

function addRow() {
   var addseason = document.getElementById("addseason").value;
   var addrecipientid = document.getElementById("addrecipientid").value;
   var additem = document.getElementById("additem").value;
   var adddesc = document.getElementById("adddesc").value;
   //adddesc=adddesc.replace('&', '\\&'));
   var addprice = document.getElementById("addprice").value;
   if (additem == "" || additem == null || additem.charAt(0) == ' '){
      alert('Item is a mandatory field.  \\n\\nPlease amend and retry.');
      return false;
   }
   if (addprice == "" || addprice == null || addprice.charAt(0) == ' ' || IsNumeric(addprice) == false){
      alert('Price is a mandatory field and can only be numeric.  \\n\\nPlease amend and retry.');
      return false;
   }
   
   var addstatusid = document.getElementById("addstatusid").value;
   var addstring = ','+addseason+','+addrecipientid+','+additem+','+adddesc+','+addprice+','+addstatusid 
   var rowcount = mygrid.getRowsNum();
   mygrid.addRow(rowcount + 1, addstring);
}
function IsNumeric(input) {
    return (input - 0) == input && input.length > 0;
}
function showtotals(originalRequest) {
   var returndata=originalRequest.responseText;
   document.getElementById("totaltablecontainer").innerHTML=returndata;
}
function setDoneStatus(userobj,userid,season) {
	var donestatus = userobj.checked;
	//alert('donestatus '+donestatus+' userid '+userid+ 'season '+season);
	if (userobj == "" || userid == null || season == null){
      alert('Item is a mandatory field.  \\n\\nPlease amend and retry.');
      return false;
   }
	var url = "xmas_totals.cgi";
	var params='season='+season+'&userid='+userid+'&donestatus='+donestatus+'&action=setdone';
	//alert('params '+params);
	var myAjax = new Ajax.Request(url,{method: 'post', parameters: params});
}
</script>
</head>
<body>
<center>
<h3>Christmas List Manager</h3>
  <table class="searchformtable">
     <tr>
     <th>Recipient</th>
     <th>Status</th>
     <th>Containing</th>
     <th>Season</th>
     </tr>
	 <tr>
	  <!-- <td class="toprowlabeltd">Recipient</td>-->
	  <td  class="toprowtdl">
            <select id="recipientid">
            <option value="" selected>All</option> 
            $recipientoptionmenuguts
            </select>
	  </td>
	  <!-- <td class="toprowlabeltd">Status</td> -->
	  <td  class="toprowtdl">
            <select id="statusid" style="width:110px;">
            <option value="" selected>All</option>
            <option value="999">All But ACQUIRED</option>
            $statusoptionmenuguts
	  </td>
	  <!-- <td class="toprowlabeltd">Containing</td> -->
	  <td  class="toprowtdl">
            <input type="text" id="likestring"  onkeypress="{if (event.keyCode==13) return reloadGrid();}"></input>
	  </td>
	  <!-- <td class="toprowlabeltd">Season</td> -->
	  <td  class="toprowtdl">
            <select id="season">
            <option value="">All</option>
            $seasonoptionmenuguts
	  </td>
	 </tr>
</table>

<!-- <span id="searchfooter"><span style="color:red;">*</span> = Recommended Minimum Search Fields</span> -->

<input type="button" class="formbutton" onclick="return reloadGrid();" value="Search"></input>
<input type="button" class="formbutton" onclick="setDefaults();" value="Reset"></input>
<!-- <input type="button" class="formbutton" name="" value="Export" onclick="mygrid.toExcel('/server/grid2excel/generate.php');"> -->
<br>
<div id="msgcontainer">
   <span id="centermessagespan"></span>
   <span id="rightmsgspan">
   <!-- <a href= 'javascript:mygrid.toPDF("https://dhtmlxgrid.appspot.com/export/pdf");'>PDF</a>&nbsp;&nbsp; -->
   <a href= "javascript:mygrid.toExcel('http://dhtmlxgrid.appspot.com/export/excel');">Export to Excel</a>
   </span>
   <span id="leftmsgspan">&nbsp;</span>
   <a href="javascript:void();" onclick="myDataProcessor.sendData();">Save Changes</a>
</div>
<div id="gridbox" style="width:840px;height:360px;background-color:transparent;"></div>
<!-- <input type="button" name="update_button" value="update" onclick="myDataProcessor.sendData();"> -->
<script>
mygrid = new dhtmlXGridObject('gridbox');
mygrid.setImagePath("/js/dhtmlxsuite/codebase/imgs/");
mygrid.setHeader(",Season,Recipient,Gift,Information,Price,Status,Delete",
null,
["text-align:center;","text-align:center;","text-align:center","text-align:center;","text-align:center;","text-align:center","text-align:center","text-align:center"]
);
mygrid.setInitWidths("0,84,84,160,300,50,110,50");
mygrid.setColAlign("left,center,left,left,left,right,center,center");
mygrid.setColTypes("ro,coro,coro,ed,txt,ed,coro,ch")
mygrid.setColSorting("int,int,str,str,str,int,str,int");
//the actual skin is set by the includes at the top of the webpage.  this just sets the css strings to load
mygrid.setSkin("dhx_skyblue");
mygrid.init();

//mygrid.attachEvent("onRowSelect", function(id,ind){fetchDetails(id,ind);});
//set the combobox values for the season selection
var seasoncombo = mygrid.getCombo(1); // index of the column to get combo object (zero relative)
$seasonoptionpopulate
//set the combobox values for the recipient selection
var recipientcombo = mygrid.getCombo(2); // index of the column to get combo object (zero relative)
$recipientoptionpopulate
//set the combobox values for the status selection
var statcombo = mygrid.getCombo(6); // index of the column to get combo object (zero relative)
$statusoptionpopulate

myDataProcessor = new dataProcessor("./xmas_dbupdateall.cgi");
//lock feed url;
myDataProcessor.setTransactionMode("POST", true);
//set mode as send-all-by-post;
myDataProcessor.setUpdateMode("off");

myDataProcessor.init(mygrid);

//link dataprocessor to the grid;
myDataProcessor.attachEvent("onAfterUpdate", function() {
	//alert('onAfterUpdate completed');
	reloadGrid();
   fetchtotals();
	return true;
});
</script>
<br>
<div id="totaltablecontainer"></div>
<br>
<table id="addrowtable">
<tr>
    <th>Season</th>
    <th>Recipient</th>
    <th>Item</th>
    <th>Description</th>
    <th>Price</th>
    <th>Status</th>
</tr>
<tr>
<td>
    <select id="addseason">
    $seasonoptionmenuguts
</td>
<td>
    <select id="addrecipientid">
    $recipientoptionmenuguts
</td>
<td><input type="text" id="additem"</input></td>
<td><input type="text" id="adddesc"</td>
<td><input type="text" id="addprice"</td></td>
<td>
    <select id="addstatusid">
    $statusoptionmenuguts
</td>
</tr>
</table>
<a href="javascript:void();" onclick="addRow();">Add Gift</a>
<br>
<!-- <input type="button" class="formbutton" onclick="return fetchtotals();" value="Show Totals"></input> -->
<br>
<a href="xmas_usernotes.cgi">Recipient Gift Notes</a>&nbsp;&nbsp;&nbsp;<a href="xmas_summary.cgi">5 Year Summary</a>
</body>
</html>
};
